@@ -20,10 +20,10 @@ from UM.Signal import Signal, signalemitter
 @signalemitter
 class SceneNode:
     """A scene node object.
-    
+
     These objects can hold a mesh and multiple children. Each node has a transformation matrix
     that maps it it's parents space to the local space (it's inverse maps local space to parent).
-    
+
     SceneNodes can be "Decorated" by adding SceneNodeDecorator objects.
     These decorators can add functionality to scene nodes.
     :sa SceneNodeDecorator
@@ -61,6 +61,9 @@ class SceneNode:
         # World transformation (from root to local)
         self._world_transformation = Matrix()  # type: Matrix
 
+        # This is used for rendering. Since we don't want to recompute it every time, we cache it in the node
+        self._cached_normal_matrix = Matrix()
+
         # Convenience "components" of the world_transformation
         self._derived_position = Vector()  # type: Vector
         self._derived_orientation = Quaternion()  # type: Quaternion
@@ -388,7 +391,7 @@ class SceneNode:
                     transformed_vertices = numpy.concatenate((transformed_vertices, tv), axis = 0)
         else:
             if self._mesh_data:
-                transformed_vertices = self._mesh_data.getTransformed(self.getWorldTransformation()).getVertices()
+                transformed_vertices = self._mesh_data.getTransformed(self.getWorldTransformation(copy=False)).getVertices()
         return transformed_vertices
 
     def getMeshDataTransformedNormals(self) -> numpy.ndarray:
@@ -408,7 +411,7 @@ class SceneNode:
                     transformed_normals = numpy.concatenate((transformed_normals, tv), axis = 0)
         else:
             if self._mesh_data:
-                transformed_normals = self._mesh_data.getTransformed(self.getWorldTransformation()).getNormals()
+                transformed_normals = self._mesh_data.getTransformed(self.getWorldTransformation(copy = False)).getNormals()
         return transformed_normals
 
     def setMeshData(self, mesh_data: Optional[MeshData]) -> None:
@@ -504,11 +507,23 @@ class SceneNode:
 
     childrenChanged = Signal()
     """Emitted whenever the list of children of this object or any child object changes.
-    
+
     :param object: The object that triggered the change.
     """
 
-    def getWorldTransformation(self) -> Matrix:
+    def _updateCachedNormalMatrix(self) -> None:
+        self._cached_normal_matrix = Matrix(self.getWorldTransformation(copy=False).getData())
+        self._cached_normal_matrix.setRow(3, [0, 0, 0, 1])
+        self._cached_normal_matrix.setColumn(3, [0, 0, 0, 1])
+        self._cached_normal_matrix.invert()
+        self._cached_normal_matrix.transpose()
+
+    def getCachedNormalMatrix(self) -> Matrix:
+        if self._cached_normal_matrix is None:
+            self._updateCachedNormalMatrix()
+        return self._cached_normal_matrix
+
+    def getWorldTransformation(self, copy = True) -> Matrix:
         """Computes and returns the transformation from world to local space.
 
         :returns: 4x4 transformation matrix
@@ -516,19 +531,21 @@ class SceneNode:
 
         if self._world_transformation is None:
             self._updateWorldTransformation()
+        if copy:
+            return self._world_transformation.copy()
+        return self._world_transformation
 
-        return self._world_transformation.copy()
-
-    def getLocalTransformation(self) -> Matrix:
+    def getLocalTransformation(self, copy = True) -> Matrix:
         """Returns the local transformation with respect to its parent. (from parent to local)
 
-        :retuns transformation 4x4 (homogenous) matrix
+        :returns transformation 4x4 (homogeneous) matrix
         """
 
         if self._transformation is None:
             self._updateLocalTransformation()
-
-        return self._transformation.copy()
+        if copy:
+            return self._transformation.copy()
+        return self._transformation
 
     def setTransformation(self, transformation: Matrix):
         self._transformation = transformation.copy() # Make a copy to ensure we never change the given transformation
@@ -544,7 +561,7 @@ class SceneNode:
 
     def rotate(self, rotation: Quaternion, transform_space: int = TransformSpace.Local) -> None:
         """Rotate the scene object (and thus its children) by given amount
-        
+
         :param rotation: :type{Quaternion} A quaternion indicating the amount of rotation.
         :param transform_space: The space relative to which to rotate. Can be any one of the constants in SceneNode::TransformSpace.
         """
@@ -566,7 +583,7 @@ class SceneNode:
 
     def setOrientation(self, orientation: Quaternion, transform_space: int = TransformSpace.Local) -> None:
         """Set the local orientation of this scene node.
-        
+
         :param orientation: :type{Quaternion} The new orientation of this scene node.
         :param transform_space: The space relative to which to rotate. Can be Local or World from SceneNode::TransformSpace.
         """
@@ -598,7 +615,7 @@ class SceneNode:
 
     def scale(self, scale: Vector, transform_space: int = TransformSpace.Local) -> None:
         """Scale the scene object (and thus its children) by given amount
-        
+
         :param scale: :type{Vector} A Vector with three scale values
         :param transform_space: The space relative to which to scale. Can be any one of the constants in SceneNode::TransformSpace.
         """
@@ -621,7 +638,7 @@ class SceneNode:
 
     def setScale(self, scale: Vector, transform_space: int = TransformSpace.Local) -> None:
         """Set the local scale value.
-        
+
         :param scale: :type{Vector} The new scale value of the scene node.
         :param transform_space: The space relative to which to rotate. Can be Local or World from SceneNode::TransformSpace.
         """
@@ -649,7 +666,7 @@ class SceneNode:
 
     def translate(self, translation: Vector, transform_space: int = TransformSpace.Local) -> None:
         """Translate the scene object (and thus its children) by given amount.
-        
+
         :param translation: :type{Vector} The amount to translate by.
         :param transform_space: The space relative to which to translate. Can be any one of the constants in SceneNode::TransformSpace.
         """
@@ -671,7 +688,7 @@ class SceneNode:
 
     def setPosition(self, position: Vector, transform_space: int = TransformSpace.Local) -> None:
         """Set the local position value.
-        
+
         :param position: The new position value of the SceneNode.
         :param transform_space: The space relative to which to rotate. Can be Local or World from SceneNode::TransformSpace.
         """
@@ -692,7 +709,7 @@ class SceneNode:
 
     def lookAt(self, target: Vector, up: Vector = Vector.Unit_Y) -> None:
         """Rotate this scene node in such a way that it is looking at target.
-        
+
         :param target: :type{Vector} The target to look at.
         :param up: :type{Vector} The vector to consider up. Defaults to Vector.Unit_Y, i.e. (0, 1, 0).
         """
@@ -720,9 +737,9 @@ class SceneNode:
         If you need to handle rendering in a special way, for example for tool handles,
         you can override this method and render the node. Return True to prevent the
         view from rendering any attached mesh data.
-        
+
         :param renderer: The renderer object to use for rendering.
-        
+
         :return: False if the view should render this node, True if we handle our own rendering.
         """
 
@@ -747,7 +764,7 @@ class SceneNode:
 
     def isSelectable(self) -> bool:
         """Get whether this SceneNode can be selected.
-        
+
         :note This will return false if isEnabled() returns false.
         """
 
@@ -755,7 +772,7 @@ class SceneNode:
 
     def setSelectable(self, select: bool) -> None:
         """Set whether this SceneNode can be selected.
-        
+
         :param select: True if this SceneNode should be selectable, False if not.
         """
 
@@ -772,7 +789,7 @@ class SceneNode:
 
     def setCalculateBoundingBox(self, calculate: bool) -> None:
         """Set whether or not to calculate the bounding box for this node.
-        
+
         :param calculate: True if the bounding box should be calculated, False if not.
         """
 
@@ -820,6 +837,7 @@ class SceneNode:
     def _updateTransformation(self) -> None:
         self._updateLocalTransformation()
         self._updateWorldTransformation()
+        self._updateCachedNormalMatrix()
 
     def _resetAABB(self) -> None:
         if not self._calculate_aabb:
@@ -832,7 +850,7 @@ class SceneNode:
 
     def _calculateAABB(self) -> None:
         if self._mesh_data:
-            aabb = self._mesh_data.getExtents(self.getWorldTransformation())
+            aabb = self._mesh_data.getExtents(self.getWorldTransformation(copy = False))
         else:  # If there is no mesh_data, use a boundingbox that encompasses the local (0,0,0)
             position = self.getWorldPosition()
             aabb = AxisAlignedBox(minimum = position, maximum = position)
