@@ -4,11 +4,12 @@
 from PyQt5.QtGui import QImage, QOpenGLFramebufferObject, QOpenGLFramebufferObjectFormat
 
 
-##  An interface for OpenGL FrameBuffer Objects.
-#
-#   This class describes a minimal interface that is expected of FrameBuffer Object
-#   classes.
 class FrameBufferObject:
+    """An interface for OpenGL FrameBuffer Objects.
+
+    This class describes a minimal interface that is expected of FrameBuffer Object
+    classes.
+    """
     def __init__(self, width: int, height: int) -> None:
         super().__init__()
 
@@ -18,21 +19,21 @@ class FrameBufferObject:
 
         self._contents = None
 
-    ##  Get the texture ID of the texture target of this FBO.
     def getTextureId(self) -> int:
+        """Get the texture ID of the texture target of this FBO."""
         return self._fbo.texture()
 
-    ##  Bind the FBO so it can be rendered to.
     def bind(self) -> None:
+        """Bind the FBO so it can be rendered to."""
         self._contents = None
         self._fbo.bind()
 
-    ##  Release the FBO so it will no longer be rendered to.
     def release(self) -> None:
+        """Release the FBO so it will no longer be rendered to."""
         self._fbo.release()
 
-    ##  Get the contents of the FBO as an image data object.
     def getContents(self) -> QImage:
+        """Get the contents of the FBO as an image data object."""
         if not self._contents:
             self._contents = self._fbo.toImage()
 
