@@ -198,9 +198,6 @@ class ShaderProgram:
 
         :note If the shader is not bound, this will bind the shader.
         """
-        if not self._shader_program:
-            return
-
         self.bind()
 
         if name not in self._attribute_indices:
@@ -210,16 +207,16 @@ class ShaderProgram:
         if attribute == -1:
             return
 
-        if type == "int":
+        if type == "vector3f":
+            self._shader_program.setAttributeBuffer(attribute, 0x1406, offset, 3, stride) #GL_FLOAT
+        elif type == "vector2f":
+            self._shader_program.setAttributeBuffer(attribute, 0x1406, offset, 2, stride)  # GL_FLOAT
+        elif type == "vector4f":
+            self._shader_program.setAttributeBuffer(attribute, 0x1406, offset, 4, stride)  # GL_FLOAT
+        elif type == "int":
             self._shader_program.setAttributeBuffer(attribute, 0x1404, offset, 1, stride) #GL_INT
         elif type == "float":
             self._shader_program.setAttributeBuffer(attribute, 0x1406, offset, 1, stride) #GL_FLOAT
-        elif type == "vector2f":
-            self._shader_program.setAttributeBuffer(attribute, 0x1406, offset, 2, stride) #GL_FLOAT
-        elif type == "vector3f":
-            self._shader_program.setAttributeBuffer(attribute, 0x1406, offset, 3, stride) #GL_FLOAT
-        elif type == "vector4f":
-            self._shader_program.setAttributeBuffer(attribute, 0x1406, offset, 4, stride) #GL_FLOAT
 
         self._shader_program.enableAttributeArray(attribute)
 
@@ -238,10 +235,10 @@ class ShaderProgram:
 
     def bind(self) -> None:
         """Bind the shader to use it for rendering."""
-        if not self._shader_program or not self._shader_program.isLinked():
+        if self._bound:
             return
 
-        if self._bound:
+        if not self._shader_program or not self._shader_program.isLinked():
             return
 
         self._shader_program.bind()
@@ -337,14 +334,15 @@ class ShaderProgram:
         elif type(value) is Color:
             self._shader_program.setUniformValue(uniform,
                 QColor(value.r * 255, value.g * 255, value.b * 255, value.a * 255))
-        elif type(value) is list and len(value) is 2:
+        elif type(value) is list and type(value[0]) is list and len(value[0]) == 4:
+            self._shader_program.setUniformValue(uniform, self._matrixToQMatrix4x4(Matrix(value)))
+        elif type(value) is list and len(value) == 2:
             self._shader_program.setUniformValue(uniform, QVector2D(value[0], value[1]))
-        elif type(value) is list and len(value) is 3:
+        elif type(value) is list and len(value) == 3:
             self._shader_program.setUniformValue(uniform, QVector3D(value[0], value[1], value[2]))
-        elif type(value) is list and len(value) is 4:
+        elif type(value) is list and len(value) == 4:
             self._shader_program.setUniformValue(uniform, QVector4D(value[0], value[1], value[2], value[3]))
-        elif type(value) is list and type(value[0]) is list and len(value[0]) is 2:
+        elif type(value) is list and type(value[0]) is list and len(value[0]) == 2:
             self._shader_program.setUniformValueArray(uniform, [QVector2D(i[0], i[1]) for i in value])
         else:
             self._shader_program.setUniformValue(uniform, value)
-
